# This file is automatically generated, DO NOT EDIT
# fmt: off

from os.path import abspath, join, dirname
_root = abspath(dirname(__file__))

# runtime dependencies
import wpilib._init_wpilib
import wpilib.geometry._init_geometry
import wpilib.interfaces._init_interfaces
import wpilib._impl._init_wpilibc
import hal._init_wpiHal
import wpiutil._init_wpiutil
import _pyntcore._init_ntcore



#!/usr/bin/env python3
"""
    A custom documentation generator because sphinx-apidoc doesn't quite do
    what I want it to. The autosummary thing is really close to what I want.
    However, the documentation for how to use it is lacking.
    
    This is designed to generate docs for our flat import structure in a
    manner that is friendly to the readthedocs theme.
"""

import os
import sys
import inspect

from os.path import abspath, join, dirname, exists

mod_doc = """
%(header)s

.. automodule:: %(module)s
    :members:

.. autosummary::
    %(cls_names)s

.. toctree::
    :hidden:
    
    %(cls_files)s

"""

cls_doc = """
%(header)s

.. automodule:: %(clsmodname)s
    :members:
    :undoc-members:
    :show-inheritance:
"""


def heading(name, c):
    return "%s\n%s" % (name, c * len(name))


def write_if_changed(fname, contents):

    try:
        with open(fname, "r") as fp:
            old_contents = fp.read()
    except:
        old_contents = ""

    if old_contents != contents:
        with open(fname, "w") as fp:
            fp.write(contents)


def gen_package(mod):

    # TODO: This might break if there is more than one class per file.. but
    #       since WPILib is emulating java, this probably won't happen...

    name = mod.__name__

    docdir = abspath(join(dirname(__file__), name))
    pkgrst = abspath(join(dirname(__file__), "%s.rst" % name))

    old_files = {}
    if exists(docdir):
        for fname in os.listdir(docdir):
            old_files[join(docdir, fname)] = True
    else:
        os.mkdir(docdir)

    classes = []

    for clsname, cls in inspect.getmembers(mod, inspect.isclass):
        fname = join(docdir, "%s.rst" % clsname)
        clsmodname = cls.__module__
        write_if_changed(
            fname,
            cls_doc
            % {
                "modname": name,
                "header": heading(clsname, "-"),
                "clsname": clsname,
                "clsmodname": clsmodname,
            },
        )

        old_files.pop(fname, None)
        classes.append((clsmodname, clsname))

    classes = sorted(classes)

    # Create toctree

    cls_names = ["%s.%s" % (clsmodule, clsname) for clsmodule, clsname in classes]
    cls_files = ["%s/%s" % (name, clsname) for clsmodule, clsname in classes]

    write_if_changed(
        pkgrst,
        mod_doc
        % {
            "header": heading(name + " Package", "="),
            "module": name,
            "cls_names": "\n    ".join(cls_names),
            "cls_files": "\n    ".join(cls_files),
        },
    )

    old_files.pop(pkgrst, None)

    # delete any files that were not written, since this is an autogenerated directory
    for fname in old_files.keys():
        os.unlink(fname)


def main():

    sys.path.insert(0, abspath(join(dirname(__file__), "..", "wpilib")))
    sys.path.insert(0, abspath(join(dirname(__file__), "..", "hal-base")))
    sys.path.insert(0, abspath(join(dirname(__file__), "..", "hal-sim")))

    import wpilib
    import wpilib.buttons
    import wpilib.command
    import wpilib.drive
    import wpilib.interfaces
    import wpilib.shuffleboard

    gen_package(wpilib)
    gen_package(wpilib.buttons)
    gen_package(wpilib.command)
    gen_package(wpilib.drive)
    gen_package(wpilib.interfaces)
    gen_package(wpilib.shuffleboard)


if __name__ == "__main__":
    main()



import hal
import wpilib
import logging
import threading


class Main:
    """
        Executes the robot code using the currently installed HAL (this is probably not what you want unless you're on the roboRIO)
    """

    def __init__(self, parser):
        pass

    def run(self, options, robot_class, **static_options):
        return robot_class.main(robot_class)


class RobotStarter:
    def __init__(self):
        self.logger = logging.getLogger("robotpy")
        self.robot = None
        self.suppressExitWarning = False

    def run(self, robot_cls: wpilib.RobotBase) -> bool:
        if hal.hasMain():
            rval = [False]

            def _start():
                try:
                    rval[0] = self.start(robot_cls)
                finally:
                    hal.exitMain()

            th = threading.Thread(target=_start, name="RobotThread", daemon=True)
            th.start()
            hal.runMain()
            self.suppressExitWarning = True
            if self.robot:
                try:
                    self.robot.endCompetition()
                except:
                    self.logger.warn("endCompetition raised an exception")

            th.join(1)
            if th.is_alive():
                self.logger.warn("robot thread didn't die, crash may occur next!")
        else:
            return self.start(robot_cls)

    def start(self, robot_cls: wpilib.RobotBase) -> bool:

        import hal
        import wpilib

        DriverStation = wpilib.DriverStation

        hal.report(
            hal.tResourceType.kResourceType_Language,
            hal.tInstances.kLanguage_Python,
            0,
            wpilib.__version__,
        )

        try:
            self.robot = robot_cls()
        except:
            DriverStation.reportError(
                "Unhandled exception instantiating robot " + robot_cls.__name__, True
            )
            DriverStation.reportWarning("Robots should not quit, but yours did!", False)
            DriverStation.reportError(
                "Could not instantiate robot " + robot_cls.__name__ + "!", False
            )
            return False

        # TODO: Add a check to see if the user forgot to call super().__init__()
        # if not hasattr(robot, "_RobotBase__initialized"):
        #     logger.error(
        #         "If your robot class has an __init__ function, it must call super().__init__()!"
        #     )
        #     return False

        if not wpilib.RobotBase.isSimulation():
            try:
                with open("/tmp/frc_versions/FRC_Lib_Version.ini", "w") as fp:
                    fp.write("RobotPy %s" % wpilib.__version__)
            except:
                DriverStation.reportError(
                    "Could not write FRC version file to disk", True
                )

        try:
            self.robot.startCompetition()
        except KeyboardInterrupt:
            self.robot = None
            self.logger.exception(
                "THIS IS NOT AN ERROR: The user hit CTRL-C to kill the robot"
            )
            self.logger.info("Exiting because of keyboard interrupt")
            return True
        except:
            self.robot = None

            DriverStation.reportError("Unhandled exception", True)
            DriverStation.reportWarning("Robots should not quit, but yours did!", False)
            DriverStation.reportError(
                "The startCompetition() method (or methods called by it) should have handled the exception above.",
                False,
            )
            return False
        else:
            self.robot = None
            if self.suppressExitWarning:
                self.logger.info("Robot code exited")
                return True
            else:
                # startCompetition never returns unless exception occurs....
                DriverStation.reportWarning(
                    "Robots should not quit, but yours did!", False
                )
                DriverStation.reportError(
                    "Unexpected return from startCompetition() method.", False
                )
                return False



from . import _init_controller

# autogenerated by 'robotpy-build create-imports wpilib.controller wpilib.controller._controller'
from ._controller import (
    ArmFeedforward,
    ElevatorFeedforwardFeet,
    ElevatorFeedforwardMeters,
    PIDController,
    RamseteController,
    SimpleMotorFeedforwardFeet,
    SimpleMotorFeedforwardMeters,
)

__all__ = [
    "ArmFeedforward",
    "ElevatorFeedforwardFeet",
    "ElevatorFeedforwardMeters",
    "PIDController",
    "RamseteController",
    "SimpleMotorFeedforwardFeet",
    "SimpleMotorFeedforwardMeters",
]



from . import _init_kinematics

# autogenerated by 'robotpy-build create-imports wpilib.kinematics wpilib.kinematics._kinematics'
from ._kinematics import (
    ChassisSpeeds,
    DifferentialDriveKinematics,
    DifferentialDriveOdometry,
    DifferentialDriveWheelSpeeds,
    MecanumDriveKinematics,
    MecanumDriveOdometry,
    MecanumDriveWheelSpeeds,
)

__all__ = [
    "ChassisSpeeds",
    "DifferentialDriveKinematics",
    "DifferentialDriveOdometry",
    "DifferentialDriveWheelSpeeds",
    "MecanumDriveKinematics",
    "MecanumDriveOdometry",
    "MecanumDriveWheelSpeeds",
]



# novalidate

import logging
import pprint

# TODO: Make these configurable
log_datefmt = "%H:%M:%S"
log_format = "%(asctime)s:%(msecs)03d %(levelname)-8s: %(name)-20s: %(message)s"


def configure_logging(verbose):

    formatter = VerboseExceptionFormatter(fmt=log_format, datefmt=log_datefmt)

    # console logging
    handler = logging.StreamHandler()
    handler.setFormatter(formatter)

    logging.root.addHandler(handler)
    logging.root.setLevel(logging.DEBUG if verbose else logging.INFO)


MAX_VARS_LINES = 30
MAX_LINE_LENGTH = 100


class VerboseExceptionFormatter(logging.Formatter):
    """
    Taken from http://word.bitly.com/post/69080588278/logging-locals
    """

    def __init__(self, log_locals_on_exception=True, *args, **kwargs):
        self._log_locals = log_locals_on_exception
        super(VerboseExceptionFormatter, self).__init__(*args, **kwargs)

    def formatException(self, exc_info):
        # First get the original formatted exception.
        exc_text = super(VerboseExceptionFormatter, self).formatException(exc_info)
        if not self._log_locals:
            return exc_text
        # Now we're going to format and add the locals information.
        output_lines = [exc_text, "\n"]

        # Retrieve locals from the innermost exception
        exc = exc_info[1]
        while exc.__cause__:
            exc = exc.__cause__

        tb = exc.__traceback__  # This is the outermost frame of the traceback.
        if tb:  # this should always be true, but sometimes someone messes up
            while tb.tb_next:
                tb = tb.tb_next  # Zoom to the innermost frame.
            output_lines.append("Locals at innermost frame:\n")
            locals_text = pprint.pformat(tb.tb_frame.f_locals, indent=2)
            locals_lines = locals_text.split("\n")
            if len(locals_lines) > MAX_VARS_LINES:
                locals_lines = locals_lines[:MAX_VARS_LINES]
                locals_lines[-1] = "..."
            output_lines.extend(
                line[: MAX_LINE_LENGTH - 3] + "..."
                if len(line) > MAX_LINE_LENGTH
                else line
                for line in locals_lines
            )
            output_lines.append("\n")
        return "\n".join(output_lines)
